#!/usr/bin/python3
# coding: utf-8
#
#       nodesBondingDDialog.py
#
#       節点結合の設定Dialog
#
#   20/08/16    新規作成
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk

import os
import GtkParts
import pyFistr
import geometryFunc as geo

ls = "\n"

#-------------------
#  dialogApp class
#-------------------
class dialogApp:

    def __init__(self, nodeGrps, bondNN):
        self.builder = Gtk.Builder()
        path = os.getenv("easyIstrPath") + os.sep + "python" + os.sep
        self.builder.add_from_file(path + "nodesBondingDDialog.glade")
        self.dialog = self.builder.get_object("dialog1")
        self.dialog.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        self.setGtkObject()
        #変数の設定
        self.stat = "CANCEL"
        self.data = []              #meshHeaderData
        self.nodeGrps = nodeGrps
        self.bondNN = bondNN
        self.nodeDict = {}
        #初期化
        self.initialize()
    #
    #  show
    #--------
    def show(self):
        """ dialog表示"""
        self.dialog.run()
        return [self.stat, self.data]

    #
    #  close
    #---------
    def close(self, *args):
        """ dialog閉じる"""
        self.dialog.destroy()
    
    #------- event handler ---------------
    #OK
    def onOk(self, event):
        self.stat = "OK"
        self.data = self.nodesBonding()     #節点結合
        self.close()
    #キャンセル
    def onCancel(self, event):
        self.stat = "CANCEL"
        self.data = []
        self.close()
    #設定
    def onSet(self, event):
        self.setBondPair()
    #戻す
    def onReturn(self, event):
        self.returnBonrPair()
    #----------------------------------------

    #
    #  setGtkObject
    #---------------
    def setGtkObject(self):
        self.tree_grps = self.builder.get_object("tree_grps")
        self.tree_bond = self.builder.get_object("tree_bond")

    #
    #  initialise
    #-------------
    def initialize(self):
        """ 初期化"""
        #nodeGrp名をtreeListにセット
        noGrpNames = self.getUnsetGrpNames()
        self.treeListGrps = GtkParts.treeList(self.tree_grps)
        self.treeListGrps.create()
        self.treeListGrps.setItems(noGrpNames)
        #bondするnodegrp名をセット
        bonds = []
        for bond in self.bondNN:
            name = "bondNN " + " ".join(bond)
            bonds.append(name)
        self.treeListBond = GtkParts.treeList(self.tree_bond)
        self.treeListBond.create()
        self.treeListBond.setItems(bonds)

    #
    #  getUnsetGrpNames
    def getUnsetGrpNames(self):
        """ 未セットのnodeGroup名を取得して返す"""
        setNames = []
        for bond in self.bondNN:
            setNames += bond
        names = []
        for name in self.nodeGrps:
            if not name in setNames:
                names.append(name)
        return names

    #
    #  setBondPair
    def setBondPair(self):
        """ 選択したpairを結合側に移動する"""
        #選択itemを取得
        selNames = self.treeListGrps.getSelectedItems()
        if len(selNames) != 2:
            title = _("エラー")
            mess = _("nodeGroupは、ペアとなる2ヶのnodeGroupを選択してください。")
            self.errDialog(title, mess)
        #表示から選択Itemを除く
        allNames = self.treeListGrps.getAllItems()
        names = []
        for name in allNames:
            if not name in selNames:
                names.append(name)
        self.treeListGrps.setItems(names)
        #bond側に追加する
        selNames.sort()
        newName = "bondNN " + " ".join(selNames)
        allNames = self.treeListBond.getAllItems()
        allNames += [newName]
        allNames.sort()
        self.treeListBond.setItems(allNames)
        self.treeListBond.selectItems([newName])

    #
    #  returnBonrPair
    def returnBonrPair(self):
        """ 選択したbondPairを戻す"""
        #bond側の選択Itemを取得
        selNames = self.treeListBond.getSelectedItems()
        allNames = self.treeListBond.getAllItems()
        names = []
        for name in allNames:
            if not name in selNames:
                names.append(name)
        self.treeListBond.setItems(names)
        #nodeGrp側に追加
        selGrps = []
        for pairName in selNames:
            words = pairName.split()
            selGrps += words[1:3]
        allGrps = self.treeListGrps.getAllItems()
        allGrps += selGrps
        allGrps.sort()
        self.treeListGrps.setItems(allGrps)
        self.treeListGrps.selectItems(selGrps)

    #
    #  nodesBonding
    #----------------
    def nodesBonding(self):
        """ 節点を結合したmeshHeaderDataを作成する。"""
        #メッシュ情報を取得
        lines = pyFistr.readFistrModelMsh()
        meshHeaderData = pyFistr.getHeaderNumData(lines)
        self.createNodeLocationDict(meshHeaderData)
        #bondDataを削除
        meshHeaderData = self.deleteNodesBondingData(meshHeaderData)
        #設定内容を取得
        bondConts = self.treeListBond.getAllItems()
        addHeader = []
        for bond in bondConts:
            words = bond.split()
            if words[0] == "bondNN":
                #bond nodeToNodeの作成
                addHeader += self.createEquationNodeToNode(words[1:3], meshHeaderData)
        #meshHeaderDataに追加
        #  挿入位置を取得
        st = len(meshHeaderData) - 2
        i = len(meshHeaderData) - 1
        while i >= 0:
            header = meshHeaderData[i][0]
            if header[:len("!END")].upper() == "!END":
                st = i
                break
        #  挿入する
        meshHeaderData = meshHeaderData[:st] + addHeader + meshHeaderData[st:]
        #書き込み
        lines = pyFistr.createLinesFistrModelMshHeader(meshHeaderData)
        self.writeFileLines("FistrModel.msh", lines)
        return meshHeaderData

    #
    #  createNodeLocationDict
    def createNodeLocationDict(self, meshHeaderData):
        """ 節点Noをキーとして座標が取得できる辞書作成"""
        for headerData in meshHeaderData:
            header = headerData[0]
            words = pyFistr.deleteSp(header).split(",")
            if words[0].upper() == "!NODE":
                for nodeData in headerData[1]:
                    nodeNo = nodeData[0]
                    loc = nodeData[1:]
                    self.nodeDict[nodeNo] = loc

    #
    #  deleteNodesBondingData
    def deleteNodesBondingData(self, meshHeaderData):
        """ nodesBondingData(EQUATION)を削除する"""
        newHeaderData = []
        flag = 0
        for headerData in meshHeaderData:
            header = headerData[0]
            if header[:len("#bondNN")] == "#bondNN":
                flag = 1
            else:
                if flag == 0:
                    newHeaderData.append(headerData)
                else:
                    flag = 0
        return newHeaderData

    #
    #  createEquationNodeToNode
    def createEquationNodeToNode(self, grps, meshHeaderData):
        """ nodeToNodeのEquationを作成する"""
        #nodeGrpのdataを取得
        nodeGrps = []
        count = 0
        for headerData in meshHeaderData:
            header = headerData[0]
            words = pyFistr.deleteSp(header).split(",")
            if words[0].upper() == "!NGROUP":
                name = pyFistr.getValName(header, "NGRP")
                if name in grps:
                    nodeGrps.append([name, headerData[1]])
                    count += 1
                    if count == 2:
                        break
        nodeGrps.sort()
        # if len(nodeGrps[0][1]) == len(nodeGrps[1][1]):
        #     #同数の場合
        #     addHeader = []
        #     headerData = ["#bondNN, " + nodeGrps[0][0] + ", " + nodeGrps[1][0] + ls, []]
        #     addHeader.append(headerData)
        #     header = "!EQUATION" + ls
        #     data = ["2, 0.0" + ls]
        #     data += [nodeGrps[0][0] + ", 1, 1, " + nodeGrps[1][0] + ", 1, -1" + ls]
        #     data += ["2, 0.0" + ls]
        #     data += [nodeGrps[0][0] + ", 2, 1, " + nodeGrps[1][0] + ", 2, -1" + ls]
        #     data += ["2, 0.0" + ls]
        #     data += [nodeGrps[0][0] + ", 3, 1, " + nodeGrps[1][0] + ", 3, -1" + ls]
        #     addHeader.append([header, data])

        #EQUATIONを作成
        pairNodes = self.getBondPairNodes(nodeGrps, meshHeaderData)
        addHeader = []
        headerData = ["#bondNN, " + nodeGrps[0][0] + ", " + nodeGrps[1][0] + ls, []]
        addHeader.append(headerData)
        header = "!EQUATION" + ls
        data = []
        for pairNode in pairNodes:
            node0 = str(pairNode[0])
            node1 = str(pairNode[1])
            data += ["2, 0.0" + ls]
            data += [node0 + ", 1, 1, " + node1 + ", 1, -1" + ls]
            data += ["2, 0.0" + ls]
            data += [node0 + ", 2, 1, " + node1 + ", 2, -1" + ls]
            data += ["2, 0.0" + ls]
            data += [node0 + ", 3, 1, " + node1 + ", 3, -1" + ls]
        addHeader.append([header, data])
        return addHeader

    #
    #  getBondPairNodes
    def getBondPairNodes(self, nodeGrps, meshHeaderData):
        """ bondするpairの節点を取得する"""
        #nodeGrps[0]側を作成
        pairNodes = []
        for nodeNo in nodeGrps[0][1]:
            disps = []
            for pairNode in nodeGrps[1][1]:
                disp = geo.length(self.nodeDict[nodeNo], self.nodeDict[pairNode])
                disps.append([disp, pairNode])
            disps.sort()
            pairNodes.append([nodeNo, disps[0][1]])
        #nodeGrps[1]側をチェック
        setNodes = map(lambda x: x[1], pairNodes)
        unsetNodes = set(nodeGrps[1][1]).difference(setNodes)
        for nodeNo in unsetNodes:
            disps = []
            for pairNode in nodeGrps[0][1]:
                disp = geo.length(self.nodeDict[nodeNo], self.nodeDict[pairNode])
                disps.append([disp, pairNode])
            disps.sort()
            #pairNodes.append([disps[0][1], nodeNo])
            pairNodes.append([nodeNo, disps[0][1]])
        return pairNodes

    def writeFileLines(self, fileName, lines):
        """ file書き込み"""
        f = open(fileName, "w")
        f.writelines(lines)
        f.close()

    def errDialog(self, title, message):
        dlg = Gtk.MessageDialog(
                parent=self.dialog,
                message_type=Gtk.MessageType.ERROR,
                buttons=Gtk.ButtonsType.OK)
        dlg.set_title(title)
        mess1 = message.split(ls)[0]
        mess2 = ls.join(message.split(ls)[1:])
        dlg.set_markup(mess1)
        dlg.format_secondary_text(mess2)
        #dummy_ans = dlg.run()
        dlg.run() 
        dlg.destroy()

